function X = BLbendrec2D( coeffs, bendletSystem)
%
% BLbendrec2D: Bendlet reconstruction of 2D data.
%
% Input:
%
%         coeffs: bendlet coefficients of size N x M x k.
%  bendletSystem: Bendlet system whose size matches coeffs, i.e. N x M with
%                 k elements.
%
% Output:
%
%      X: N x M array containing signal represented by coeffs. 
%
% Example:
%
% X = double(imread('Data/Snowman.png'));
% bendletSystem =  BLgetBendletSystem2D( size(X,1), size(X,2));
% coeffs = BLbenddec2D( X, bendletSystem);
% Xtilde = BLbendrec2D( coeffs, bendletSystem);

    if nargin < 2
        error('Not enough input parameters!');
    end

    %initialise reconstructed data
    Xfreq = zeros(size(coeffs,1),size(coeffs,2));
    
    % perform reconstruction
    for j = 1:bendletSystem.nBendlets;
      Xfreq_j = ...
        fft2(fftshift(coeffs(:,:,j))) .* bendletSystem.bendlets(:,:,j);
      Xfreq = Xfreq + Xfreq_j;
    end

    % apply weights of dual functions
    X = ifft2(ifftshift((1./bendletSystem.dualFrameWeights) .* Xfreq));

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Copyright (c) 2016, Philipp Petersen and Christian Lessig.
%
%  If you use or mention this code in a publication please cite 
%  C. Lessig, P. Petersen, M. Schfer, Bendlets: A Second Order Shearlet
%    Transform with Bent Elements.
% 
%  This file is part of BendLab: A Bendlet Toolbox.
% 
%  BendLab is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
% 
%  BendLab is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License for more details.
% 
%  You should have received a copy of the GNU General Public License
%  along with BendLab.  If not, see <http://www.gnu.org/licenses/>.
