function bendletsystem = ...
    BLgetBendletSystem2DSparseConv( rows, cols, nScales, ...
                                    shears, bendings, a_scale, ...
                                    sth, cones)
%
% BLgetBendletSystem2DSparseConv: Computes a 2D bendlet system where a 
%                sparse representation is employed for the bendlets
%                and coefficients are computed using convolution in the 
%                spatial domain.
%
% Input:
%
%                  rows: Number of rows.
%                  cols: Number of columns.
%               nScales: Number of scales of the desired bendlet system.
%                        Has to be >= 1.
%                shears: A 1xnScales sized array, specifying the level of
%                        shearing occuring on each scale. Each entry of
%                        shearLevels has to be >= 0. a value shears = [1 2 3 4]
%                        means that on scale 1 there is one shear in each
%                        direction on scale 1, two scales in each
%                        direction on scale 2, etc.
%              bendings: A 1xnScales sized array, specifying the level of
%                        bending occuring on each scale. Each entry of
%                        bendings has to be >= 0. The usage is the same as
%                        with shears.
%               a_scale: Alpha value for the scaling of the bendlet
%                        system.
%                   sth: threshold for clamping non-zero values
%                 cones: cones for which to compute system 
%
%  nScales, shears, bendings, a_scale, sth, cones are set automatically if 
%    not specified, i.e. there are default parameters (depending on rows, 
%    cols)
%
% Output:
%
%         bendletsystem: A structure containing the specified bendlet system.   
%             .bendlets: Cell array of N 2D bendlets in the spatial domain where X and Y   
%                        denote the size of each shearlet.
%       .bendletsOffset: Offset of support of bendlet in domain
%         .bendletsSupp: Size of support window of bendlet.
%            .nBendlets: Number of bendlets in the
%                        bendletsystem.shearlets array. This number
%                        also describes the redundancy of the system.
%          .bendletIdxs: A Nx4 array, specifying each shearlet in the system
%                        in the format [cone scale shearing bending] where N
%                        denotes the number of shearlets. The lowpass bendlet is 
%                        indexed by [0 0 0 0].

    %% check input arguments
    if nargin < 2
        error('Not enough input parameters!');
    end
    if nargin < 3
      if cols ~= rows
        error('Not enough input parameters!');
      end
    end

    % set default parameters if not given
    defval( 'nScales', round(log(rows)/log(2.0) - 5.0));
    defval( 'shears', genShearsDefault(nScales,[]));
    defval( 'bendings', genBendingsDefault(nScales,[]));
    defval( 'a_scale', 0.335);
    defval( 'sth', 100*eps)
    defval( 'cones', [1, 2]);

    %% make B-spline lowpass filter
    
    lengthPsi = numel( wavefun('db8',nScales));
    
    diffPhi = 12;
    lengthPhi = floor( lengthPsi / diffPhi);
    phi = ones(1,floor(lengthPhi));
    for u = 1:diffPhi
       phi = conv(phi, ones(1,floor(lengthPhi)));
    end


    %% construct all bendlets
   
    bendlets = {};
    bendlets_offsets = [];
    bendlets_supp = [];
    dualFrameWeights = zeros( rows, cols);
    
    idx = 1;
    for cone = cones
        for scale = 1:nScales
            for shear = -shears(scale):shears(scale)
                for bend = -bendings(scale):bendings(scale)
                    
                    psi = BLbendPsi( lengthPhi, diffPhi, scale, nScales, ...
                                     rows, cols, ...
                                     shear/max((shears(scale)),1), ...
                                     bend/max((bendings(scale)),1), ...
                                     a_scale);
                    if 2 == cone
                        psi = rot90(psi);
                    end
                    
                    % find tight window
                    [psi_win, supp_r, supp_c, offset_r, offset_c] = ...
                                                          findWindow( psi);
                    bendlets{idx} = psi_win;
                    bendlets_offsets(end+1,:) = [offset_r, offset_c];
                    bendlets_supp(end+1,:) = [supp_r, supp_c];
                    
                    bendletIdxs(idx, 1:4) = [cone, scale, shear, bend];
                    idx = idx+1;
                end
            end
            disp( sprintf( 'Finished scale %i', scale));
        end
    end
    
    %% add scaling function
    
    low = phi.'*phi;
    [a1,a2] = size(low);
    % pad in all directions
    low = padarray(low, floor([(rows-a1)/2 (cols-a2)/2]), 0, 'pre');
    low = padarray(low, ceil([(rows-a1)/2 (cols-a2)/2]), 0, 'post');
    low = low/norm(low(:));
    % find tight window 
    [phi_win, supp_r, supp_c, offset_r, offset_c] = findWindow( low);
    bendlets{idx} = phi_win;
    bendlets_offsets(end+1,:) = [offset_r, offset_c];
    bendlets_supp(end+1,:) = [supp_r, supp_c];
    bendletIdxs(idx, 1:4) = [0, 0, 0, 0];
    

    %% Construct final system
   
    bendletsystem = struct;
    bendletsystem.bendlets = bendlets;
    bendletsystem.bendletsOffset = bendlets_offsets;
    bendletsystem.bendletsSupp = bendlets_supp;
    bendletsystem.nBendlets = idx;
    bendletsystem.bendletIdxs = bendletIdxs;

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [psi_win, supp_r, supp_c, offset_r, offset_c] = findWindow( psi)

    [i,j] = find( abs(psi) > 10^(-10));
    
    % support of molecule
    supp_r = [min(i), max(i)];
    supp_c = [min(j), max(j)];
    
    psi_win = psi( max(i):-1:min(i) , max(j):-1:min(j));
    
    offset_r = (size(psi,1) - max(i) - min(i)) / 2;
    offset_r = ceil( offset_r);
    
    offset_c = (size(psi,2) - max(j) - min(j)) / 2;
    offset_c = ceil( offset_c);
    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function shears = genShearsDefault( nScales, shears)

    if( nScales > 0)
        shears = [max(1.0,ceil( log(nScales)/log(2))) shears];
        shears = genShearsDefault( nScales-1, shears);
    end
    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function bendings = genBendingsDefault( nScales, bendings)

    if( nScales > 0)
        bendings = [max(1.0,ceil( log(nScales)/log(2))) bendings];
        bendings = genShearsDefault( nScales-1, bendings);
    end
    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Copyright (c) 2016, Philipp Petersen and Christian Lessig.
%
%  If you use or mention this code in a publication please cite 
%  C. Lessig, P. Petersen, M. Schfer, Bendlets: A Second Order Shearlet
%    Transform with Bent Elements.
% 
%  This file is part of BendLab: A Bendlet Toolbox.
% 
%  BendLab is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
% 
%  BendLab is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License for more details.
% 
%  You should have received a copy of the GNU General Public License
%  along with BendLab.  If not, see <http://www.gnu.org/licenses/>.

